@@ -1,5 +1,4 @@
 import hashlib
-import io
 import logging
 import os
 from collections import defaultdict
@@ -13,16 +12,16 @@ from struct import pack, unpack
 import zstandard as zstd
 from PIL import Image
 
+from modlunky2.assets.constants import KNOWN_TEXTURES_V1
 from modlunky2.assets.exc import NonSiblingAsset
 
 from .chacha import Key, chacha, hash_filepath
 from .constants import (BANK_ALIGNMENT, DEFAULT_COMPRESSION_LEVEL,
                         FILENAMES_TO_FILEPATHS, KNOWN_FILEPATHS,
-                        PNG_NAMES_TO_DDS_NAMES, PNGS)
-from .converters import dds_to_png, png_to_dds
+                        PNG_NAMES_TO_DDS_NAMES, DDS_PNGS)
+from .converters import dds_to_png, png_to_dds, rgba_to_png
 from .exc import FileConflict, MissingAsset, MultipleMatchingAssets
 
-
 logger = logging.getLogger("modlunky2")
 
 
@@ -57,15 +56,15 @@ class ExeAssetBlock:
     @property
     def total_size(self):
         return (
-            8                    # 8 bytes for header (asset_len, filepath_len)
+            8  # 8 bytes for header (asset_len, filepath_len)
             + self.filepath_len  # The size of the filepath
-            + 1                  # The byte for whether the asset is encrypted
-            + self.asset_len     # The size of the asset
+            + 1  # The byte for whether the asset is encrypted
+            + self.asset_len  # The size of the asset
         )
 
     @classmethod
     def from_exe_handle(cls, exe_handle):
-        """ From an exe file handle, construct an AssetInfo object.
+        """From an exe file handle, construct an AssetInfo object.
 
         The exe handle should already be seeked to the appropriate position to
         begin reading.
@@ -103,7 +102,9 @@ class ExeAssetBlock:
 
     def write_data(self, exe_handle, data):
         if self.asset_len != len(data):
-            raise RuntimeError(f"Data passed was size {len(data)}, expected {self.asset_len}")
+            raise RuntimeError(
+                f"Data passed was size {len(data)}, expected {self.asset_len}"
+            )
 
         exe_handle.write(pack("<II", self.data_len, self.filepath_len))
         exe_handle.write(self.filepath_hash)
@@ -128,8 +129,12 @@ class ExeAsset:
         self.data = handle.read(self.asset_block.asset_len)
 
     def extract(
-        self, extract_dir: Path, compressed_dir: Path, key: Key,
-        compression_level=DEFAULT_COMPRESSION_LEVEL
+        self,
+        extract_dir: Path,
+        compressed_dir: Path,
+        key: Key,
+        compression_level=DEFAULT_COMPRESSION_LEVEL,
+        recompress=True,
     ):
         if not self.filepath:
             raise RuntimeError("Asset doesn't have filepath.")
@@ -151,31 +156,33 @@ class ExeAsset:
                 cctx = zstd.ZstdDecompressor()
                 self.data = cctx.decompress(self.data)
 
-                # Recompress at higher compression level to give
-                # better chance of assets fitting in binary
-                logger.info("Storing compressed asset %s...", compressed_filepath)
-                with compressed_filepath.open("wb") as compressed_file:
-                    cctx = zstd.ZstdCompressor(level=compression_level)
-                    compressed_data = cctx.compress(self.data)
-                    compressed_file.write(compressed_data)
+                if recompress:
+                    # Recompress at higher compression level to give
+                    # better chance of assets fitting in binary
+                    logger.info("Storing compressed asset %s...", compressed_filepath)
+                    with compressed_filepath.open("wb") as compressed_file:
+                        cctx = zstd.ZstdCompressor(level=compression_level)
+                        compressed_data = cctx.compress(self.data)
+                        compressed_file.write(compressed_data)
 
             except Exception:  # pylint: disable=broad-except
                 logger.exception("Failed compression")
                 return None
 
+        if self.filepath in KNOWN_TEXTURES_V1:
+            self.data = rgba_to_png(self.data)
+        elif self.filepath in DDS_PNGS:
+            self.data = dds_to_png(self.data)
 
-        if self.filepath in PNGS:
-            image = Image.open(io.BytesIO(self.data))
-            self.data = dds_to_png(image)
-
-        # Get a hash of the the uncompressed file to be used
-        # to detect if the source file changed
-        md5sum = hashlib.md5(self.data).hexdigest()
-        with md5sum_filepath.open("w") as md5sum_file:
-            md5sum_file.write(md5sum)
+        if recompress:
+            # Get a hash of the the uncompressed file to be used
+            # to detect if the source file changed
+            md5sum = hashlib.md5(self.data).hexdigest()
+            with md5sum_filepath.open("w") as md5sum_file:
+                md5sum_file.write(md5sum)
 
         logger.info("Storing asset %s...", filepath)
-        if self.filepath in PNGS:
+        if self.filepath in DDS_PNGS:
             filepath = filepath.with_suffix(".png")
 
         with filepath.open("wb") as asset_file:
@@ -253,8 +260,12 @@ class AssetStore:
             logger.exception("Failed Extraction")
 
     def extract(
-        self, extract_dir, compressed_dir,
-        compression_level=DEFAULT_COMPRESSION_LEVEL
+        self,
+        extract_dir,
+        compressed_dir,
+        compression_level=DEFAULT_COMPRESSION_LEVEL,
+        max_workers=max(os.cpu_count() - 2, 1),
+        recompress=True,
     ):
         unextracted = []
         for asset in self.assets:
@@ -265,7 +276,7 @@ class AssetStore:
 
             asset.load_data(self.exe_handle)
 
-        with ThreadPoolExecutor() as pool:
+        with ThreadPoolExecutor(max_workers=max_workers) as pool:
             futures = [
                 pool.submit(
                     self._extract_single,
@@ -274,6 +285,7 @@ class AssetStore:
                     compressed_dir,
                     self.key,
                     compression_level,
+                    recompress,
                 )
                 for asset in self.assets
                 if asset.filepath
@@ -297,9 +309,9 @@ class AssetStore:
                 data = chacha(asset.filepath.encode(), data, self.key)
 
             logger.info("Packing file %s", asset.disk_asset.asset_path)
-            self.exe_handle.write(pack(
-                "<II", asset.asset_block.data_len, asset.asset_block.filepath_len
-            ))
+            self.exe_handle.write(
+                pack("<II", asset.asset_block.data_len, asset.asset_block.filepath_len)
+            )
             self.exe_handle.write(asset.asset_block.filepath_hash)
             self.exe_handle.write(pack("<b", asset.asset_block.is_encrypted))
             self.exe_handle.write(data)
@@ -324,11 +336,17 @@ class AssetStore:
             )
 
     def repackage(
-        self, search_dirs, fallback_dir, compressed_dir,
-        compression_level=DEFAULT_COMPRESSION_LEVEL
+        self,
+        search_dirs,
+        fallback_dir,
+        compressed_dir,
+        compression_level=DEFAULT_COMPRESSION_LEVEL,
     ):
         disk_bundle = DiskBundle.from_dirs(
-            self.assets, search_dirs, fallback_dir, compressed_dir,
+            self.assets,
+            search_dirs,
+            fallback_dir,
+            compressed_dir,
         )
         disk_bundle.compress_if_needed(compression_level=compression_level)
 
@@ -353,7 +371,9 @@ class AssetStore:
             #
             # Padding is between 1 and 32 bytes
             if disk_asset.asset_path.suffix == ".bank":
-                padding = BANK_ALIGNMENT - asset.asset_block.asset_offset % BANK_ALIGNMENT
+                padding = (
+                    BANK_ALIGNMENT - asset.asset_block.asset_offset % BANK_ALIGNMENT
+                )
                 asset.asset_block.filepath_len += padding
                 asset.asset_block.asset_offset += padding
 
@@ -390,14 +410,16 @@ class DiskBundle:
             dirs[:] = [d for d in dirs if d not in [".compressed"]]
 
             for file_ in files:
-                # For PNGS we need to convert to actual name to test
+                # For DDS_PNGS we need to convert to actual name to test
                 # for validity of files
                 real_name = PNG_NAMES_TO_DDS_NAMES.get(file_, file_)
                 if real_name not in FILENAMES_TO_FILEPATHS:
                     continue
 
                 if file_ in out_files:
-                    raise MultipleMatchingAssets(f"Found {file_} multiple times in {search_dir}")
+                    raise MultipleMatchingAssets(
+                        f"Found {file_} multiple times in {search_dir}"
+                    )
 
                 out_files[file_] = Path(root) / file_
 
@@ -405,22 +427,28 @@ class DiskBundle:
 
     @classmethod
     def from_dirs(
-        cls, exe_assets, search_dirs, fallback_dir, compressed_dir,
-        resolution_policy=ResolutionPolicy.RaiseError
+        cls,
+        exe_assets,
+        search_dirs,
+        fallback_dir,
+        compressed_dir,
+        resolution_policy=ResolutionPolicy.RaiseError,
     ):
 
         modfiles_by_filename = defaultdict(list)
         disk_assets = {}
 
         for search_dir in search_dirs:
-            for file_, file_path in cls.get_files_from_search_dir(Path(search_dir)).items():
+            for file_, file_path in cls.get_files_from_search_dir(
+                Path(search_dir)
+            ).items():
                 modfiles_by_filename[file_].append(file_path)
 
         for asset in exe_assets:
             if asset.filepath is None:
                 continue
             filepath = Path(asset.filepath)
-            if asset.filepath in PNGS:
+            if asset.filepath in DDS_PNGS:
                 filepath = filepath.with_suffix(".png")
             modpack_files = modfiles_by_filename.get(filepath.name)
 
@@ -436,7 +464,10 @@ class DiskBundle:
                 )
                 continue
 
-            if resolution_policy == ResolutionPolicy.RaiseError and len(modpack_files) >= 2:
+            if (
+                resolution_policy == ResolutionPolicy.RaiseError
+                and len(modpack_files) >= 2
+            ):
                 raise FileConflict(
                     f"{filepath} found in multiple packs: {', '.join(map(str, modpack_files))}"
                 )
@@ -454,8 +485,12 @@ class DiskBundle:
 
         return cls(disk_assets)
 
-    def compress_if_needed(self, compression_level=DEFAULT_COMPRESSION_LEVEL):
-        with ThreadPoolExecutor() as pool:
+    def compress_if_needed(
+        self,
+        compression_level=DEFAULT_COMPRESSION_LEVEL,
+        max_workers=max(os.cpu_count() - 2, 1),
+    ):
+        with ThreadPoolExecutor(max_workers=max_workers) as pool:
             futures = [
                 pool.submit(disk_asset.compress, compression_level)
                 for disk_asset in self.disk_assets.values()
@@ -501,17 +536,23 @@ class DiskAsset:
 
     @property
     def real_suffix(self):
-        if self.exe_asset.filepath in PNGS:
+        if self.exe_asset.filepath in DDS_PNGS:
             return ".DDS"
         return Path(self.exe_asset.filepath).suffix
 
     @property
     def compressed_path(self):
-        return self.compressed_dir / f"{self.rel_asset_path.with_suffix(self.real_suffix)}.zst"
+        return (
+            self.compressed_dir
+            / f"{self.rel_asset_path.with_suffix(self.real_suffix)}.zst"
+        )
 
     @property
     def md5sum_path(self):
-        return self.compressed_dir / f"{self.rel_asset_path.with_suffix(self.real_suffix)}.md5sum"
+        return (
+            self.compressed_dir
+            / f"{self.rel_asset_path.with_suffix(self.real_suffix)}.md5sum"
+        )
 
     def needs_compression(self):
         if not self.exe_asset.asset_block.is_encrypted:
